﻿using System;
using System.Collections.Generic;
using System.Linq;

using Foundation;
using Syncfusion.SfChart.iOS;
using Syncfusion.SfChart.XForms;
using Syncfusion.SfChart.XForms.iOS;
using Syncfusion.SfChart.XForms.iOS.Renderers;
using UIKit;

namespace StackedBarDashes.iOS
{
    // The UIApplicationDelegate for the application. This class is responsible for launching the 
    // User Interface of the application, as well as listening (and optionally responding) to 
    // application events from iOS.
    [Register("AppDelegate")]
    public partial class AppDelegate : global::Xamarin.Forms.Platform.iOS.FormsApplicationDelegate
    {
        

        //
        // This method is invoked when the application has loaded and is ready to run. In this 
        // method you should instantiate the window, load the UI into it and then make the window
        // visible.
        //
        // You have 17 seconds to return from this method, or iOS will terminate your application.
        //
        public override bool FinishedLaunching(UIApplication app, NSDictionary options)
        {
            global::Xamarin.Forms.Forms.Init();
            LoadApplication(new App());
            SfChartRenderer.Init();
            Xamarin.Forms.Forms.ViewInitialized += Forms_ViewInitialized;
            return base.FinishedLaunching(app, options);
        }

		void Forms_ViewInitialized(object sender, Xamarin.Forms.ViewInitializedEventArgs e)
		{
			SFChart nativeChart = e.NativeView as SFChart;

			SfChart formsChart = e.View as SfChart;

			if (nativeChart is SFChart)
			{
                nativeChart.DataSource = new CustomDataSource(nativeChart, formsChart);

				for (int i = 0; i < (nativeChart as SFChart).Series.Count; i++)
				{
                    if (nativeChart.Series[i] is SFStackingBarSeries)
					{
						nativeChart.Series.RemoveAt(i);

						if (nativeChart.DataSource != null)
						{
                            NativeStackingBarSeriesExt customBarSeries = (nativeChart.DataSource as CustomDataSource).GetSeries(nativeChart, i) as NativeStackingBarSeriesExt;
							nativeChart.Series.Insert(i, customBarSeries);
						}
					}
				}
			}

			
		}

		public class CustomDataSource : ChartDataSource
		{
			public SfChart Formchart
			{
				get;
				set;
			}

			public CustomDataSource(SFChart native, SfChart chart) : base(native, chart)
			{
				Formchart = chart;
			}

			public override SFSeries GetSeries(SFChart chart, nint index)
			{
				ChartSeries chartSeries = this.Formchart.Series[(int)index];

				if (chartSeries is StackingBarExt)
				{
                    NativeStackingBarSeriesExt barSeries = new NativeStackingBarSeriesExt(chartSeries as StackingBarExt);

                    List<string> properties = SfChartRenderer.GetPropertiesChanged(typeof(StackingBarExt), chartSeries);

					foreach (var name in properties)
					{
                        ChartSeriesMapping.OnStackingBarSeriesPropertiesChanged(name, (StackingBarExt)chartSeries, barSeries);
					}

					var nativeSeries = barSeries;

					SfChartRenderer.SetNativeObject(typeof(ChartSeries), chartSeries, nativeSeries);

					return nativeSeries;
				}

				return base.GetSeries(chart, index);

			}
		}
    }
}
